Stan Math Library: stan/math/prim/scal/prob/neg_binomial_cdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

neg_binomial_cdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_NEG_BINOMIAL_CDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_NEG_BINOMIAL_CDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
9 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
10 #include <stan/math/prim/scal/fun/constants.hpp>
11 #include <stan/math/prim/scal/fun/value_of.hpp>
12 #include <stan/math/prim/scal/fun/binomial_coefficient_log.hpp>
13 #include <stan/math/prim/scal/fun/multiply_log.hpp>
14 #include <stan/math/prim/scal/fun/digamma.hpp>
15 #include <stan/math/prim/scal/fun/lgamma.hpp>
16 #include <stan/math/prim/scal/fun/lbeta.hpp>
17 #include <stan/math/prim/scal/meta/include_summand.hpp>
18 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
19 #include <stan/math/prim/scal/fun/grad_reg_inc_beta.hpp>
20 #include <stan/math/prim/scal/fun/inc_beta.hpp>
21 #include <boost/math/special_functions/digamma.hpp>
22 #include <boost/random/negative_binomial_distribution.hpp>
23 #include <boost/random/variate_generator.hpp>
24 #include <cmath>
25 #include <limits>
26
27 namespace stan {
28
29 namespace math {
30
31 template <typename T_n, typename T_shape,
32 typename T_inv_scale>
33 typename return_type<T_shape, T_inv_scale>::type
34 neg_binomial_cdf_log(const T_n& n, const T_shape& alpha,
35 const T_inv_scale& beta) {
36 static const char* function("stan::math::neg_binomial_cdf_log");
37 typedef typename stan::partials_return_type<T_n, T_shape,
38 T_inv_scale>::type
39 T_partials_return;
40
41 using stan::math::check_positive_finite;
42 using stan::math::check_nonnegative;
43 using stan::math::check_consistent_sizes;
44 using stan::math::include_summand;
45
46 // Ensure non-zero arugment lengths
47 if (!(stan::length(n) && stan::length(alpha) && stan::length(beta)))
48 return 0.0;
49
50 T_partials_return P(0.0);
51
52 // Validate arguments
53 check_positive_finite(function, "Shape parameter", alpha);
54 check_positive_finite(function, "Inverse scale parameter", beta);
55 check_consistent_sizes(function,
56 "Failures variable", n,
57 "Shape parameter", alpha,
58 "Inverse scale parameter", beta);
59
60 // Wrap arguments in vector views
61 VectorView<const T_n> n_vec(n);
62 VectorView<const T_shape> alpha_vec(alpha);
63 VectorView<const T_inv_scale> beta_vec(beta);
64 size_t size = max_size(n, alpha, beta);
65
66 // Compute vectorized cdf_log and gradient
67 using stan::math::value_of;
68 using stan::math::inc_beta;
69 using stan::math::digamma;
70 using stan::math::lbeta;
71 using std::exp;
72 using std::pow;
73 using std::log;
74 using std::exp;
75
76
77 OperandsAndPartials<T_shape, T_inv_scale>
78 operands_and_partials(alpha, beta);
79
80 // Explicit return for extreme values
81 // The gradients are technically ill-defined, but treated as zero
82 for (size_t i = 0; i < stan::length(n); i++) {
83 if (value_of(n_vec[i]) < 0)
84 return operands_and_partials.value(stan::math::negative_infinity());
85 }
86
87 // Cache a few expensive function calls if alpha is a parameter
88 VectorBuilder<!is_constant_struct<T_shape>::value,
89 T_partials_return, T_shape>
90 digammaN_vec(stan::length(alpha));
91 VectorBuilder<!is_constant_struct<T_shape>::value,
92 T_partials_return, T_shape>
93 digammaAlpha_vec(stan::length(alpha));
94 VectorBuilder<!is_constant_struct<T_shape>::value,
95 T_partials_return, T_shape>
96 digammaSum_vec(stan::length(alpha));
97
98 if (!is_constant_struct<T_shape>::value) {
99 for (size_t i = 0; i < stan::length(alpha); i++) {
100 const T_partials_return n_dbl = value_of(n_vec[i]);
101 const T_partials_return alpha_dbl = value_of(alpha_vec[i]);
102
103 digammaN_vec[i] = digamma(n_dbl + 1);
104 digammaAlpha_vec[i] = digamma(alpha_dbl);
105 digammaSum_vec[i] = digamma(n_dbl + alpha_dbl + 1);
106 }
107 }
108
109 for (size_t i = 0; i < size; i++) {
110 // Explicit results for extreme values
111 // The gradients are technically ill-defined, but treated as zero
112 if (value_of(n_vec[i]) == std::numeric_limits<int>::max())
113 return operands_and_partials.value(0.0);
114
115 const T_partials_return n_dbl = value_of(n_vec[i]);
116 const T_partials_return alpha_dbl = value_of(alpha_vec[i]);
117 const T_partials_return beta_dbl = value_of(beta_vec[i]);
118 const T_partials_return p_dbl = beta_dbl / (1.0 + beta_dbl);
119 const T_partials_return d_dbl = 1.0 / ( (1.0 + beta_dbl)
120 * (1.0 + beta_dbl) );
121 const T_partials_return Pi = inc_beta(alpha_dbl, n_dbl + 1.0, p_dbl);
122 const T_partials_return beta_func = exp(lbeta(n_dbl + 1, alpha_dbl));
123
124
125 P += log(Pi);
126
127 if (!is_constant_struct<T_shape>::value) {
128 T_partials_return g1 = 0;
129 T_partials_return g2 = 0;
130
131 stan::math::grad_reg_inc_beta(g1, g2, alpha_dbl,
132 n_dbl + 1, p_dbl,
133 digammaAlpha_vec[i],
134 digammaN_vec[i],
135 digammaSum_vec[i],
136 beta_func);
137 operands_and_partials.d_x1[i] += g1 / Pi;
138 }
139 if (!is_constant_struct<T_inv_scale>::value)
140 operands_and_partials.d_x2[i] += d_dbl * pow(1-p_dbl, n_dbl)
141 * pow(p_dbl, alpha_dbl-1) / beta_func / Pi;
142 }
143
144 return operands_and_partials.value(P);
145 }
146 }
147 }
148 #endif
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
lgamma.hpp
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::lbeta
fvar< T > lbeta(const fvar< T > &x1, const fvar< T > &x2)
Definition: lbeta.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
binomial_coefficient_log.hpp
is_constant_struct.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::math::neg_binomial_cdf_log
return_type< T_shape, T_inv_scale >::type neg_binomial_cdf_log(const T_n &n, const T_shape &alpha, const T_inv_scale &beta)
Definition: neg_binomial_cdf_log.hpp:34
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::inc_beta
fvar< T > inc_beta(const fvar< T > &a, const fvar< T > &b, const fvar< T > &x)
Definition: inc_beta.hpp:20
stan::partials_return_type
Definition: partials_return_type.hpp:16
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
stan::math::max
int max(const std::vector< int > &x)
Returns the maximum coefficient in the specified column vector.
Definition: max.hpp:21
partials_return_type.hpp
OperandsAndPartials.hpp
grad_reg_inc_beta.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::size
int size(const std::vector< T > &x)
Return the size of the specified standard vector.
Definition: size.hpp:17
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
lbeta.hpp
inc_beta.hpp
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::math::grad_reg_inc_beta
void grad_reg_inc_beta(T &g1, T &g2, T a, T b, T z, T digammaA, T digammaB, T digammaSum, T betaAB)
Definition: grad_reg_inc_beta.hpp:14
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::negative_infinity
double negative_infinity()
Return negative infinity.
Definition: constants.hpp:132
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.